from uuid import uuid4 as uuid
from time import time
import hashlib
import string
import subprocess
import secrets
from base64 import b64encode as benc


class Challenge():
    def __init__(self, time_limit=120):
        self.api_keys = dict()
        self.valid_hashes = dict()
        self.time_limit = time_limit

    def get_api_key(self):
        ak = str(uuid())
        self.api_keys[ak] = int(time())
        return ak

    def check_api_key(self, key):
        t = int(time())
        if key not in self.api_keys:
            return False
        if t - self.api_keys[key] >= self.time_limit:
            del self.api_keys[key]
            return False
        return True

    def sandbox(self, file_contents):
        key, pycode = file_contents[2:38].decode(), file_contents[39:]
        md5sum = hashlib.md5()
        md5sum.update(file_contents)
        md5sum = md5sum.hexdigest()

        if not self.check_api_key(key):
            return False, md5sum

        pycode = ''.join(filter(lambda x: x in string.printable,
                                pycode.decode('utf-8', errors='ignore')))

        for _ in range(1000):
            secret = secrets.token_hex(32)
            progout = subprocess.getoutput(
                f'python3.8 runcode2.py {benc(pycode.encode()).decode()} {secret}')
            progout, token = progout[:-64], progout[-64:]
            if token != secret:
                return False, md5sum

        self.valid_hashes[md5sum] = key
        return True, md5sum

    def runcode_ceo(self, file_contents):
        key, pycode = file_contents[2:38].decode(), file_contents[39:]
        md5sum = hashlib.md5()
        md5sum.update(file_contents)
        md5sum = md5sum.hexdigest()

        if not self.check_api_key(key):
            return False, md5sum
        if md5sum not in self.valid_hashes or self.valid_hashes[md5sum] != key:
            return False, md5sum
        del self.valid_hashes[md5sum]

        pycode = ''.join(filter(lambda x: x in string.printable,
                                pycode.decode('utf-8', errors='ignore')))
        progout = subprocess.getoutput(
            f'python3.8 runcode2.py {benc(pycode.encode()).decode()}')
        print(progout)

        return True, progout


if __name__ == '__main__':
    ch = Challenge()
    key = ch.get_api_key()
    myprogram = b'# ' + key.encode() + b'''
print("hello there") '''
    print(myprogram.decode())
    print("#########")
    print(ch.sandbox(myprogram))
    print(ch.runcode_ceo(myprogram))
